"""
See: Campbell Scientific CR1000 Manual Section 7.8.5.2.2.
"""
import numpy as np


def wind_vector_components(windspd, winddir):
    """Decompose scalar or list/array wind direction and speed data into the
    corresponding horizontal and vertical direction components and speed
    vector.

    Inputs can be scalar or arrays.
    """
    dir_rad = np.deg2rad(winddir)
    spd_arr = np.array(windspd)
    V_e = spd_arr * np.sin(dir_rad)
    V_n = spd_arr * np.cos(dir_rad)
    U_spd = np.sqrt(pow(V_e, 2) + pow(V_n, 2))
    return V_e, V_n, U_spd


def wind_vector_degrees(vector_east, vector_north):
    """Re-compose horizontal (east/west) and vertical (north/south) vector
    components into wind direction in degrees.

    Inputs can be scalar or arrays.
    """
    rads = np.arctan2(vector_east, vector_north)
    winddir = np.rad2deg(rads)
    if isinstance(winddir, np.ndarray):
        winddir[np.less(winddir, 0)] += 360
    elif winddir < 0:
        winddir += 360
    return winddir % 360


def mean_wind_vector(windspd, winddir):
    V_e, V_n, V_spd = wind_vector_components(windspd, winddir)
    avg_dir = wind_vector_degrees(np.mean(V_e), np.mean(V_n))

    return avg_dir, np.mean(V_spd)
